@@ -5,11 +5,12 @@
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  * Copyright 2001, Apple Computer, Inc.
+ * Copyright (c) 2005-2007 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXInt.h,v 1.3.2.14 2006/09/11 14:41:16 das Exp $
+ * RCS: @(#) $Id: tkMacOSXInt.h,v 1.3.2.22 2007/08/23 10:45:15 das Exp $
  */
 
 #ifndef _TKMACINT
@@ -32,17 +33,19 @@
 #endif
 
 struct TkWindowPrivate {
-    TkWindow *winPtr;     	/* Ptr to tk window or NULL if Pixmap */
-    CGrafPtr  grafPtr;
+    TkWindow *winPtr;		/* Ptr to tk window or NULL if Pixmap */
+    CGrafPtr grafPtr;
+    CGContextRef context;
     ControlRef rootControl;
-    int xOff;	       		/* X offset from toplevel window */
-    int yOff;		       	/* Y offset from toplevel window */
+    int xOff;			/* X offset from toplevel window */
+    int yOff;			/* Y offset from toplevel window */
     RgnHandle clipRgn;		/* Visible region of window */
-    RgnHandle aboveClipRgn;	/* Visible region of window & it's children */
+    RgnHandle aboveClipRgn;	/* Visible region of window & its children */
+    RgnHandle drawRgn;		/* Clipped drawing region */
     int referenceCount;		/* Don't delete toplevel until children are
 				 * gone. */
-    struct TkWindowPrivate *toplevel;	/* Pointer to the toplevel
-					 * datastruct. */
+    struct TkWindowPrivate *toplevel;
+				/* Pointer to the toplevel datastruct. */
     int flags;			/* Various state see defines below. */
 };
 typedef struct TkWindowPrivate MacDrawable;
@@ -54,50 +57,52 @@ typedef struct TkWindowPrivate MacDrawable;
  */
 
 typedef struct TkMacOSXWindowList {
-    struct TkMacOSXWindowList *nextPtr;	/* The next window in the list. */
-    TkWindow *winPtr;			/* This window */
+    struct TkMacOSXWindowList *nextPtr;
+				/* The next window in the list. */
+    TkWindow *winPtr;		/* This window */
 } TkMacOSXWindowList;
 
 /*
  * Defines use for the flags field of the MacDrawable data structure.
  */
- 
-#define TK_SCROLLBAR_GROW	1
-#define TK_CLIP_INVALID		2
-#define TK_HOST_EXISTS		4
-#define TK_DRAWN_UNDER_MENU	8
+
+#define TK_SCROLLBAR_GROW	0x01
+#define TK_CLIP_INVALID		0x02
+#define TK_HOST_EXISTS		0x04
+#define TK_DRAWN_UNDER_MENU	0x08
+#define TK_CLIPPED_DRAW		0x10
 
 /*
  * I am reserving TK_EMBEDDED = 0x100 in the MacDrawable flags
  * This is defined in tk.h. We need to duplicate the TK_EMBEDDED flag in the
- * TkWindow structure for the window, but in the MacWin.  This way we can
+ * TkWindow structure for the window, but in the MacWin. This way we can
  * still tell what the correct port is after the TKWindow structure has been
- * freed.  This actually happens when you bind destroy of a toplevel to
+ * freed. This actually happens when you bind destroy of a toplevel to
  * Destroy of a child.
  */
 
 /*
  * This structure is for handling Netscape-type in process
- * embedding where Tk does not control the top-level.  It contains
+ * embedding where Tk does not control the top-level. It contains
  * various functions that are needed by Mac specific routines, like
- * TkMacOSXGetDrawablePort.  The definitions of the function types
- * are in tclMac.h.
+ * TkMacOSXGetDrawablePort. The definitions of the function types
+ * are in tkMacOSX.h.
  */
 
 typedef struct {
-	Tk_MacOSXEmbedRegisterWinProc *registerWinProc;
-	Tk_MacOSXEmbedGetGrafPortProc *getPortProc;
-	Tk_MacOSXEmbedMakeContainerExistProc *containerExistProc;
-	Tk_MacOSXEmbedGetClipProc *getClipProc;
-	Tk_MacOSXEmbedGetOffsetInParentProc *getOffsetProc;
+    Tk_MacOSXEmbedRegisterWinProc *registerWinProc;
+    Tk_MacOSXEmbedGetGrafPortProc *getPortProc;
+    Tk_MacOSXEmbedMakeContainerExistProc *containerExistProc;
+    Tk_MacOSXEmbedGetClipProc *getClipProc;
+    Tk_MacOSXEmbedGetOffsetInParentProc *getOffsetProc;
 } TkMacOSXEmbedHandler;
 
-MODULE_SCOPE TkMacOSXEmbedHandler *gMacEmbedHandler;
+MODULE_SCOPE TkMacOSXEmbedHandler *tkMacOSXEmbedHandler;
 
 /*
  * Defines used for TkMacOSXInvalidateWindow
  */
- 
+
 #define TK_WINDOW_ONLY 0
 #define TK_PARENT_WINDOW 1
 
@@ -117,25 +122,13 @@ MODULE_SCOPE TkMacOSXEmbedHandler *gMacEmbedHandler;
 #define TK_BOTH_CHANGED		3
 
 /*
- * Variables shared among various Mac Tk modules but are not
- * exported to the outside world.
- */
-
-/*
- * Globals shared among Macintosh Tk
+ * Globals shared among TkAqua.
  */
 
 MODULE_SCOPE MenuHandle tkCurrentAppleMenu; /* Handle to current Apple Menu */
 MODULE_SCOPE MenuHandle tkAppleMenu;	/* Handle to default Apple Menu */
 MODULE_SCOPE MenuHandle tkFileMenu;	/* Handles to menus */
 MODULE_SCOPE MenuHandle tkEditMenu;	/* Handles to menus */
-MODULE_SCOPE RgnHandle tkMenuCascadeRgn;/* A region to clip with. */
-MODULE_SCOPE int tkUseMenuCascadeRgn;	/* If this is 1, clipping code
-					 * should intersect tkMenuCascadeRgn
-					 * before drawing occurs.
-					 * tkMenuCascadeRgn will only
-					 * be valid when the value of this
-					 * variable is 1. */
 MODULE_SCOPE int tkPictureIsOpen;	/* If this is 1, we are drawing to a
 					 * picture The clipping should then be
 					 * done relative to the bounds of the
@@ -147,29 +140,24 @@ MODULE_SCOPE int tkPictureIsOpen;	/* If this is 1, we are drawing to a
 					 * for CopyBits should also have
 					 * top,left values of 0,0
 					 */
-MODULE_SCOPE TkMacOSXWindowList *tkMacOSXWindowListPtr;
-					/* The list of toplevels */
-
+MODULE_SCOPE TkMacOSXWindowList *tkMacOSXWindowListPtr; /* List of toplevels */
 MODULE_SCOPE Tcl_Encoding TkMacOSXCarbonEncoding;
 
-MODULE_SCOPE void TkMacOSXDisplayChanged(Display *display);
-MODULE_SCOPE int TkMacOSXUseAntialiasedText(Tcl_Interp *interp, int enable);
-MODULE_SCOPE void TkMacOSXInitCarbonEvents(Tcl_Interp *interp);
-MODULE_SCOPE int TkMacOSXInitCGDrawing(Tcl_Interp *interp, int enable, int antiAlias);
-MODULE_SCOPE void TkMacOSXInitKeyboard(Tcl_Interp *interp);
-MODULE_SCOPE void TkMacOSXDefaultStartupScript(void);
-MODULE_SCOPE int TkMacOSXGenerateFocusEvent( Window window, int activeFlag);
-MODULE_SCOPE WindowClass TkMacOSXWindowClass(TkWindow *winPtr);
-MODULE_SCOPE int TkMacOSXIsWindowZoomed(TkWindow *winPtr);
-MODULE_SCOPE int TkGenerateButtonEventForXPointer(Window window);
+/*
+ * Prototypes of internal procs not in the stubs table.
+ */
 
-MODULE_SCOPE void* TkMacOSXGetNamedSymbol(const char* module, const char* symbol);
-/* Macro to abstract common use of TkMacOSXGetNamedSymbol to initialize named symbols */
-#define TkMacOSXInitNamedSymbol(module, ret, symbol, ...) \
-    static ret (* symbol)(__VA_ARGS__) = (void*)(-1L); \
-    if (symbol == (void*)(-1L)) { \
-        symbol = TkMacOSXGetNamedSymbol(STRINGIFY(module), STRINGIFY(_##symbol));\
-    }
+MODULE_SCOPE void TkMacOSXDefaultStartupScript(void);
+#if 0
+MODULE_SCOPE int XSetClipRectangles(Display *d, GC gc, int clip_x_origin,
+	int clip_y_origin, XRectangle* rectangles, int n, int ordering);
+#endif
+MODULE_SCOPE void TkpClipDrawableToRect(Display *display, Drawable d, int x,
+	int y, int width, int height);
+
+/*
+ * Include the stubbed internal platform-specific API.
+ */
 
 #include "tkIntPlatDecls.h"
 
